
; LOADER fuer CPD2

		TITLE	CPD2LOAD


		.Z80



		; Umschalten der Speicherbank von ROM -> RAM
		SET	BIT_RAMSEL,A

		; Einschalten der gelben LED
		SET	BIT_LED_GE,A
		;
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A
		;
		; Initialisierung des definierten Anfangszustandes:
		; Umschalten des Interface Transmission Protokoll (ITP) auf Lesen (RD)
		CALL	INT_MODE_RD
		;
		LD	HL,ZK_START	; Anzeige des Kaltstarttextes
		CALL	AUS_ZK
		;
		LD	HL,ZK_ANZ_BOOT	; Anzeige ZK_ANZ_BOOT
		CALL	AUS_ZK
		;
		CALL	RD_SE_CNT	; Lesen der Anzahl der Sektoren
					; der BOOT-Datei
		LD	(VAR_SECNT),A
		;
		CALL	RD_BOOT_FILE
		;
		JP	TPA










; **********************************************************
;                   LOADER Speicherzellen und Texte
; **********************************************************


VAR_SECNT:	DB	0		; Anzahl der Sektoren fuer das BOOT-File

VAR_SECNR:	DB	0		; Zaehler fuer die angezeigten Sektoren


ZK_START:
		DB	0CH
		DB	'Monitor 2011 by Ronald Daleske',0DH,0AH,0DH,0AH,0

ZK_ANZ_BOOT:
		DB	"Die BOOT-Datei wird ab Adresse 100H eingelesen",0DH,0AH,0


ZK_ANZ_SEK:
		DB	0DH,0AH,"Sektor:",0





; **********************************************************
;                   Unterprogramm-Bibliothek
; **********************************************************





; *** CONOUT ***
; BIOS-Funktion (04) : CONSOLE-Ausgabe
; CONOUT (CONsole OUTput) : Zeichen zur Console ausgeben, 
; Der im Register C befindliche Wert wird zur Console (Bildschirm) ausgegeben.
; Aufruf: CONOUT
; Uebergabeparameter: Register C = Zeichen im ASCII-Code
; Rueckgabeparameter: keine
;    
LCONOUT:
		PUSH	BC		; Retten des Zeichens in C

		; Umschalten des Interface Transmission Protokoll (ITP) auf Schreiben (WR)
		CALL	INT_MODE_WR
		;
		LD	B,SE_CONOUT
		CALL	ITP_SEND_01
		;
		POP	BC
		LD	B,C
		CALL	ITP_SEND_01
		;
		RET






; **********************************************************




; Funktion: ANZ_SEK_NR
;
; Aufruf: ANZ_SEK_NR
; Uebergabeparameter: Register C = Sektornummer
; Rueckgabeparameter: keine
; 
ANZ_SEK_NR:
		;
		LD	A,(VAR_SECNR)
		INC	A
		LD	(VAR_SECNR),A
		CP	11
		;
		JR	NZ,ANZ_SEK_NR_1
		LD	A,0
		LD	(VAR_SECNR),A
		;
		PUSH	BC
		LD	HL,ZK_ANZ_SEK	; Anzeige "Sektor:"
		CALL	AUS_ZK
		POP	BC

ANZ_SEK_NR_1:
		LD	A,C
		CALL	ByteToCONOUT
		;
		LD	C,','
		CALL	LCONOUT
		;
		RET





; Funktion: ByteToCONOUT
;
; Aufruf: ByteToCONOUT
; Uebergabeparameter: Register A = Bytewert
; Rueckgabeparameter: keine
; 
ByteToCONOUT:
		CALL	ByteToHex
		PUSH	BC
		;
		LD	C,B
		CALL	LCONOUT
		;
		POP	BC
		CALL	LCONOUT
		;
		LD	C,'H'
		CALL	LCONOUT
		;
		RET






; Funktion: ByteToHex
;
; Aufruf: ByteToHex
; Uebergabeparameter: Register A = Bytewert
; Rueckgabeparameter: Register BC - B = HWT, C = LWT
; 
ByteToHex:
		PUSH	AF
		AND	00FH
		;
		CALL	ByteToHex_A
		LD	C,A
		;
		POP	AF
		SRL	A
		SRL	A
		SRL	A
		SRL	A
		;
		CALL	ByteToHex_A
		LD	B,A
		;
		RET


ByteToHex_A:	ADD	A,'0'		; Zahl '0'
		CP	'9'+1		; '9'+1 -> 0..9 = Cyrry , A..F = not Curry
		RET	C
		;
		ADD	A,07		; 65D ('A') - 48D ('0') = 17D -> Differenz zw. '0' und 'A'
		;
		RET




; Funktion: Read BOOT File (Lesen der Anzahl der Sektoren)
;
; Aufruf: RD_BOOT_FILE
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
; 
RD_BOOT_FILE:
		;
		LD	HL,ZK_ANZ_SEK	; Anzeige "Sektor:"
		CALL	AUS_ZK
		;
		XOR	A
		LD	(VAR_SECNR),A
		;
		LD	C,A		; C=0
		;
		LD	A,(VAR_SECNT)
		LD	B,A
		;
		LD	HL,TPA
			
RD_BOOT_FILE_1:
		PUSH	BC	
		CALL	RD_BOOT_SEK
		POP	BC
		;
		PUSH	BC
		PUSH	HL
		CALL	ANZ_SEK_NR
		POP	HL
		POP	BC
		;
		INC	C
		LD	A,B
		CP	C
		;
		JR	NZ,RD_BOOT_FILE_1
		;
		RET







; Funktion: Read BOOT Sektor (Lesen der Anzahl der Sektoren)
;
; Aufruf: RD_BOOT_SEK
; Uebergabeparameter: Register C = Nummer des zu lesenden Sektors (0..VAR_SECNT-1)
;                     Register HL = Ziel-Adresse fuer den Sektor 
; Rueckgabeparameter: Register HL = Ziel-Adresse fuer weitere Sektoren
;  
RD_BOOT_SEK:
		LD	B,0
RD_BOOT_SEK_A:
		PUSH	BC
		CALL	RD_BOOT_SEA
		;
		POP	BC
		INC	B
		LD	A,B
		CP	4
		;
		JR	NZ,RD_BOOT_SEK_A
		;
		RET




; Funktion: Read Sektor-Abschnitt der BOOT-Datei
;
; Aufruf: RD_BOOT_SEA
; Uebergabeparameter: Register B = 0..7 - lfd. Nr. des 16er Blockes aus dem Sektor
;                     Register C = Nummer des zu lesenden Sektors (0..VAR_SECNT-1)
;                     Register HL = Ziel-Adresse fuer den Sektor 
; Rueckgabeparameter: Register HL = Ziel-Adresse fuer weitere Sektoren
; 
RD_BOOT_SEA:	; Register retten (Reihenfolge wichtig)
		PUSH	HL
		PUSH	BC
		;
		; Umschalten des Interface Transmission Protokoll (ITP) auf Schreiben (WR)
		CALL	INT_MODE_WR
		;
		LD	B,RE_SECTOR
		CALL	ITP_SEND_01
		;
		POP	BC
		PUSH	BC
		CALL	ITP_SEND_01		; Uebergabe lfd. Nr. des 16er Blockes aus dem Sektor 
		;
		POP	BC
		LD	B,C
		CALL	ITP_SEND_01		; Uebergabe Nummer des zu lesenden Sektors 
		;
		; Umschalten des Interface Transmission Protokoll (ITP) auf Lesen (RD)
		CALL	INT_MODE_RD
		;
		LD	B,32
		PUSH	BC
RD_BOOT_SEA_A:
		CALL	ITP_RECEIVE_01		; Abholen der Daten, Ergebnis in A
		POP	BC
		POP	HL
		LD	(HL),A	
		INC	HL
		DEC	B
		JR	Z,RD_BOOT_SEA_X
		;
		PUSH	HL
		PUSH	BC
		;
		JR	RD_BOOT_SEA_A
		
RD_BOOT_SEA_X:
		;
		RET




; Funktion: Read Sector Count (Lesen der Anzahl der Sektoren)
;
; Aufruf: RD_SE_NR
; Uebergabeparameter: keine
; Rueckgabeparameter: Register A und Speicherzelle VAR_SECNT
; 
RD_SE_CNT:
		; Umschalten des Interface Transmission Protokoll (ITP) auf Schreiben (WR)
		CALL	INT_MODE_WR
		;
		LD	B,RE_SECNT
		CALL	ITP_SEND_01
		;
		; Umschalten des Interface Transmission Protokoll (ITP) auf Lesen (RD)
		CALL	INT_MODE_RD
		;
		CALL	ITP_RECEIVE_01		; Abholen der Daten, Ergebnis in A
		;
		LD	(VAR_SECNT),A
		;
		RET





; Funktion: Ausgabe einer Zeichenkette auf den Bildschirm
;
; Aufruf: AUS_ZK
; Uebergabeparameter: Register HL = Adresse der Zeichenkette 
;                     Ende = 00H
; Rueckgabeparameter: keine
; 
AUS_ZK:		LD	A,(HL)
		OR	A
		RET	Z		;RET wenn Zeichen = 0 ist
		LD	C,A
		PUSH	HL
		CALL	LCONOUT		;Ausgabe des Zeichens in Reg.C
		POP	HL
		INC	HL
		;
		JR	AUS_ZK





		DB	"ENDE"



